#pragma once
// Hopefully this goes without saying - don't edit this file.
#include "Messenger.h" // defines EXP
#include "CmdLineSocket.h"  // for connection initiated from SA
#include "DockingBtnCtrl.h" // for docking into SA
#include <string>
#include <vector>
#include <ctime>

#define SAIPSCANSTRIPE_NUMPTS 5000

class SAConnection;
class Messenger;

class EXP SAxform
{
public:
    double T[4][4];
};

class EXP SAvector
{
public:
    double x;
    double y;
    double z;
};

class EXP SAScanStripe
{
public:
    std::vector<SAvector> pts;
    SAxform scannerXform;

    void Clear()
    {
        for (size_t i = 0; i < pts.size(); i++)
            pts.pop_back();
    }
};

class SAInstrument
{
public:
    EXP SAInstrument(Messenger& _messenger);
    EXP ~SAInstrument(void);

    EXP CmdLineSocket* GetAutoSock();
    EXP bool Logon(int saIndex, std::string interfaceName);
    EXP bool SetDistUnit(int dUnit); // BE SURE to call this after you Logon (see distunits enum below). Just set to the units you work in.
    EXP bool IsConnectedToSA();
    EXP bool DisconnectFromSA(); // should be called after you disconnect from your hardware (to prevent any errant attempt to send data to SA if any is received from hdw)
    EXP void SaveSettings();

    // docking control - you can use this to dock your dialog inside the SA window
    EXP void SetupDockingButton(CWnd* parent, CRect rect); // parent is your dialog, rect is the CRect from your (invisible) button placed where you want it in your dialog, see CSAInstSDKUserSampleDlg::OnInitDialog()
    EXP BOOL OnDockWrapperCommand(CDialog& dDlg, WPARAM wParam, LPARAM lParam); // you have to define OnCommand in your dialog, see CSAInstSDKUserSampleDlg::OnCommand

    // utilities
    EXP SAxform GetInstrumentToSAWorking(); // this is the transform of your instrument in SA's Working frame (WORLD by default). Gets updated at startup and whenever it changes.
    EXP SAxform SetFromOriginAndFixedRxyzDegrees(const SAvector& origin, const SAvector& fixedXYZRotation);
    EXP void Remove6DTargetModel(); // turn off your (optional) display of a 6D probe

    // stuff to SA
    // Connection Status
    EXP void SignalInterfaceReady();
    EXP void SignalInterfaceFailed();
    // Measurements
    EXP void SendPointToSA(SAvector& xyz, SAvector& sigma, double radialOffset, double planarOffset, bool updateOnly, std::string target, std::string group, std::string collection = "", std::vector<std::string> notes = {""});
    EXP void SendEDMTheoPtToSA(SAvector& dhv, SAxform& frame, bool is6D, double radialOffset, double planarOffset, bool updateOnly, std::string target, std::string group, std::string collection="", std::vector<std::string> notes = {""});
    EXP void SendCMMPtToSA(SAvector& xyz, SAxform& frame, double radialOffset, double planarOffset, bool updateOnly, bool sendJointValues, std::vector<double>& j, std::string target, std::string group, std::string collection/*=""*/, std::vector<std::string> notes/* = {""}*/);
    EXP void SendIJKUncertPointToSA(SAvector& xyz, SAvector& ijk, SAvector& sigma, double radialOffset, double planarOffset, bool updateOnly, std::string targ, std::string group, std::string collection = "", std::vector<std::string> notes = { "" });
    EXP void SendIJKUncertPointArrayToSA(std::vector<SAvector>& xyz, std::vector<SAvector>& ijk, std::vector<SAvector>& sigma, double radialOffset, double planarOffset, std::string firstTarget, std::string group, std::string collection = "");
    EXP void SendIJKUncertPointWithNamesArrayToSA(std::vector<SAvector>& xyz, std::vector<SAvector>& ijk, std::vector<SAvector>& sigma, std::vector<std::string> name, double radialOffset, double planarOffset, std::string group, std::string collection/*=""*/);
    EXP void Send6DPointToSA(SAvector& xyz, SAxform& frame, double timeSeconds, SAvector& sigma, double radialOffset, double planarOffset, bool updateOnly, std::string targ, std::string group, std::string collection = "", std::vector<std::string> notes = { "" });
    EXP void SendBatchofPtsToSA(std::vector<SAvector>& measPt, std::vector<SAxform>& measXform, std::vector<SAvector>& sigma, std::vector<std::string>& c, std::vector<std::string>& g, std::vector<std::string>& t, double radialOffset = 0.0, double planarOffset = 0.0, std::vector<std::string> notes = { "" });
    EXP void SendFrameDataToSA(SAxform& frame, double timeSeconds, bool updateOnly, bool overWriteExisting, bool drawProbe, std::string name, std::string collection = "", std::vector<std::string> notes = { "" }, double radialOffset = 0.0, double planarOffset = 0.0);
    EXP void SendMultipleFrameDataToSA(std::vector<SAxform>& frames, std::vector<double>& timesSeconds, std::string setName, double radialOffset = 0.0, double planarOffset = 0.0);
    EXP void SendMultipleInstFramesToSA(std::vector<SAxform>& frames, std::vector<std::string>& names, std::vector<std::time_t>& times); // this is for INSTRUMENT network transforms, NOT measured transforms - only applies to e.g. photogrammetry cameras or "indoor gps" transmitters
    EXP void SendPointSetTrackingToSA(std::vector<SAvector>& measPt, std::vector<std::string>& name); // for TransTrack with pts - "Transform in 6D Using a Batch of Points from an Instrument"
    EXP void SendScanStripeCloudToSA(std::vector<SAScanStripe>& cloud, std::string name, std::string collection = "");
    EXP void SendPointCloudToSA(std::vector<SAvector>& cloud, std::string name, std::string collection = "");
    // Features
    EXP void SendLineToSA(SAvector& a, SAvector& b, char DrawArrow, std::string name, std::string collection = "", std::vector<std::string> notes = { "" });
    EXP void SendCircleToSA(SAxform& CircleTrans, double Radius, double Span, std::string Name, std::string collection = "", std::vector<std::string> notes = { "" });
    EXP void SendSphereToSA(SAvector& Center, double Radius, std::string Name, std::string collection = "", std::vector<std::string> notes = { "" });
    EXP void SendCylinderToSA(SAvector& a, SAvector& b, double Diameter, std::string name, std::string collection = "", std::vector<std::string> notes = { "" });
    EXP void SendPlaneToSA(SAxform& PlaneTrans, std::vector<SAvector> pts, std::string name, std::string collection = "", std::vector<std::string> notes = { "" });
    // Report Items
    EXP void SendInstrumentHistoryUpdate(std::string title, std::vector<std::string> bodyText, std::string type = "");
    // Run an MP
    EXP void RunMPByName(std::string mpPath);

    // stuff from SA
    // sent via the Messenger, you derive your own from it so you can override all the "...FromSA" handlers

    // units
    enum distunits
    {
        du_ft = 0,
        du_in = 1,
        du_m = 2,
        du_cm = 3,
        du_mm = 4
    };
#define MAX_DISTUNITS 5

    friend SAConnection;

private:
    SAConnection& sa;
    Messenger& messenger;
    CDockingBtnCtrl dockingButton;
};
